/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Fireclam.
 *
 * The Initial Developer of the Original Code is
 * Christof Efkemann.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


var cc = Components.classes;
var ci = Components.interfaces;
var cr = Components.results;
var cu = Components.utils;

cu.import("resource://gre/modules/XPCOMUtils.jsm");

var stringService = cc["@mozilla.org/intl/stringbundle;1"].getService(ci.nsIStringBundleService);
var strings = stringService.createBundle("chrome://fireclam/locale/fireclam.properties");
var osString  = cc["@mozilla.org/xre/app-info;1"].getService(ci.nsIXULRuntime).OS;


function getString(name)
{
    return strings.GetStringFromName(name);
}


function formatString(name, args)
{
    return strings.formatStringFromName(name, args, args.length);
}


function scanThread(prog, args, file, thread, logFile)
{
    this.prog = prog;
    this.args = args;
    this.file = file;
    this.thread = thread;
    this.logFile = logFile;
}


scanThread.prototype = {
    run: function()
    {
	var exitcode;

	try {
	    var clamScanFile = cc["@mozilla.org/file/local;1"].createInstance(ci.nsILocalFile);
	    clamScanFile.initWithPath(this.prog);

	    var clamScanProc = cc["@mozilla.org/process/util;1"].createInstance(ci.nsIProcess);
	    clamScanProc.init(clamScanFile);
	    clamScanProc.run(true, this.args, this.args.length);
	    exitcode = clamScanProc.exitValue;
	}
	catch (e) {
	    exitcode = -127;
	    this.file = this.prog;
	}

	var main = cc["@mozilla.org/thread-manager;1"].getService().mainThread;
	main.dispatch(new resultThread(this.file, exitcode, this.thread, this.logFile), main.DISPATCH_NORMAL);
    },

    QueryInterface: function(iid)
    {
	if (iid.equals(ci.nsIRunnable) ||
	    iid.equals(ci.nsISupports)) {
	    return this;
	}
	throw cr.NS_ERROR_NO_INTERFACE;
    }
};


function resultThread(file, exitcode, bgThread, logFile)
{
    this.file = file;
    this.exitcode = exitcode;
    this.bgThread = bgThread;
    this.logFile = logFile;
}


resultThread.prototype = {
    run: function()
    {
	if (this.bgThread != null)
	    this.bgThread.shutdown();
	var promptService = cc["@mozilla.org/embedcomp/prompt-service;1"].getService(ci.nsIPromptService);
	var fstream = cc["@mozilla.org/network/file-input-stream;1"].createInstance(ci.nsIFileInputStream);
	var cstream = cc["@mozilla.org/intl/converter-input-stream;1"].createInstance(ci.nsIConverterInputStream);
	var logMessage;

	fstream.init(this.logFile, -1, 0, 0);
	cstream.init(fstream, "UTF-8", 0, 0);
	let (str = {}) {
	    cstream.readString(-1, str);
	    logMessage = str.value;
	}
	cstream.close();  // also closes fstream

	switch (this.exitcode) {
	case 0:
	    /* file is clean, nothing to do */
	    break;
		
	case 1:
	    /* file is infected, tell user */
	    promptService.alert(null,
				getString("virus_report_title"),
				formatString("virus_report_message", [this.file.path, logMessage]));
	    break;

	case 50:
	    /* database error */
	    promptService.alert(null,
				getString("problem_title"),
				formatString("database_error_message", [this.file.path]));
	    break;

	case -127:
	    /* special return: executable not found */
	    promptService.alert(null,
				getString("problem_title"),
				formatString("missing_executable_message", [this.file.path]));
	    break;

	default:
	    /* some other error occurred */
	    promptService.alert(null,
				getString("problem_title"),
				formatString("problem_message", [this.file.path, this.exitcode]));
	}

	/* remove log file */
	this.logFile.remove(false);
    },

    QueryInterface: function(iid)
    {
	if (iid.equals(ci.nsIRunnable) ||
	    iid.equals(ci.nsISupports)) {
	    return this;
	}
	throw cr.NS_ERROR_NO_INTERFACE;
    }
};


function fireclamService()
{
    var defaultPath = "/usr/bin/clamscan";
    var defaultDBPath = "/var/lib/clamav";

    if (osString == "WINNT") {
        try {
	    var wrk = cc["@mozilla.org/windows-registry-key;1"].
		createInstance(ci.nsIWindowsRegKey);
	    wrk.open(wrk.ROOT_KEY_LOCAL_MACHINE,
		     "SOFTWARE\\ClamWin",
		     wrk.ACCESS_READ);
	    var iniPath = wrk.readStringValue("Path") + "\\ClamWin.conf";

	    var iniFile = cc["@mozilla.org/file/local;1"].createInstance(ci.nsILocalFile);
	    iniFile.initWithPath(iniPath);

	    var fis = cc["@mozilla.org/network/file-input-stream;1"].createInstance(ci.nsIFileInputStream);
	    fis.init(iniFile, 0x01, 0444, 0);
	    fis = fis.QueryInterface(ci.nsILineInputStream);

	    var reClamScan = /^clamscan = (.*)$/;
	    var reDBPath = /^database = (.*)$/;
	    var line = {};
	    var cont;
	    var result;

	    do {
		cont = fis.readLine(line);
		if ((result = line.value.match(reClamScan))) {
		    defaultPath = result[1];
		}
		if ((result = line.value.match(reDBPath))) {
		    defaultDBPath = result[1];
		}
	    } while (cont);
	    fis.close();
	}
	catch (e)
	{
	}
    }
    var prefService = cc["@mozilla.org/preferences-service;1"].getService(ci.nsIPrefService);
    var branch = prefService.getDefaultBranch("extensions.fireclam.");
    branch.setCharPref("clamavexecutable", defaultPath);
    branch.setCharPref("dbpath", defaultDBPath);

    var obsService = cc["@mozilla.org/observer-service;1"].getService(ci.nsIObserverService);
    obsService.addObserver(this, "dl-done", false);
}


fireclamService.prototype = {
    classDescription: "fireclam Service",
    classID: Components.ID("{13E3DFE2-FC31-4358-8273-8C0A3FC69003}"),
    contractID: "@efkemann.net/fireclam;1",
    _xpcom_categories: [{ category: "app-startup", service: true }],

    QueryInterface: XPCOMUtils.generateQI([ci.nsIObserver]),

    observe: function(subject, topic, data)
    {
	if (topic == "dl-done") {
	    var download = subject.QueryInterface(ci.nsIDownload);
	    var file = download.targetFile;
	    this.scanFile(file);
	}
    },

    scanFile: function(file)
    {
	var prefService = cc["@mozilla.org/preferences-service;1"].getService(ci.nsIPrefService);
	var dirService = cc["@mozilla.org/file/directory_service;1"].getService(ci.nsIProperties);
	var branch = prefService.getBranch("extensions.fireclam.");
	var path = branch.getCharPref("clamavexecutable");
	var dbpath = branch.getCharPref("dbpath");
	var logFile = dirService.get("TmpD", ci.nsIFile);
	var uFile;
	var uLogFile;

	logFile.append("fireclam.tmp");
	logFile.createUnique(ci.nsIFile.NORMAL_FILE_TYPE, 0600);

	if (osString != "WINNT") {
	    var converter = cc["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(ci.nsIScriptableUnicodeConverter);
	    converter.charset = "UTF-8";
	    uFile = converter.ConvertFromUnicode(file.path) + converter.Finish();
	    uLogFile = converter.ConvertFromUnicode(logFile.path) + converter.Finish();
	}
	else {
	    uFile = file.path;
	    uLogFile = logFile.path;
	}

	var args = ["--no-summary", "--quiet", "-l", uLogFile, "-d", dbpath, uFile];
	if (XPCOMUtils.generateNSGetFactory) {
	    // Firefox 4: cannot use background thread
	    var main = cc["@mozilla.org/thread-manager;1"].getService().mainThread;
	    main.dispatch(new scanThread(path, args, file, null, logFile), main.DISPATCH_NORMAL);
	}
	else {
	    var bgThread = cc["@mozilla.org/thread-manager;1"].getService().newThread(0);
	    bgThread.dispatch(new scanThread(path, args, file, bgThread, logFile), bgThread.DISPATCH_NORMAL);
	}
    },
};


var components = [fireclamService];

if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
else
    var NSGetModule = XPCOMUtils.generateNSGetModule(components);

// Local Variables:
// coding: iso-8859-1
// c-basic-offset: 4
// End:
